#include <cassert>
#include <iostream>
#include <string>
#include <math.h>
#include "Genome.h"
#include "valid_seq.h"
#include "ltqnorm.h"

//used for finding the random length of a string
#define RAND_HIGH 1000	
#define CHAR_TOTAL 1000

using namespace std;

///! Some params that are (mostly) worthless, yet used in Genome.h
unsigned int BISULFITE = 0;
unsigned int ATOG = 0;
unsigned char INT2BASE[16];
unsigned char bs_CONVERSION[256];
unsigned char gen_CONVERSION[256];
int MER_SIZE=9;
unsigned int BUFFER_SIZE=1000000;
unsigned int HASH_SIZE=0;
unsigned int MAX_HASH_SIZE=0;
int MAX_GAP = 3;
float ALIGN_SCORE=0;

float GAP = -2;
float MATCH = 1;
float MISMATCH = -1;

float ALIGN_SCORES[256][4];
int ADD_SIZE=3;
int BIN_SIZE=16;

char* adaptor;
float MIN_ADAPTOR_DIFF = 0.75;
unsigned int MIN_CHOPPED_BASES = 4;
// ! End Genome.h files

const char* genome_file;
const char* output_file;
const char* pos_file=NULL;
unsigned int output_type=0;
		/* output_type is: 
		 * 0 = all
		 * 1 = fasta
		 * 2 = fq
		 * 3 = fa&fq
		 * 4 = prb&int
		 */
bool t_fasta=true;
bool t_fq=true;
bool t_int=true;
bool t_prb=true;
unsigned int num_gen=1000;
double perc=0;
int SEQ_LENGTH=0;
int mean=0;
int sd = 0;
int VERBOSE=0;
bool RAND=0;
bool EQ_SPACE=0;


/*****************************************************
 * These next functions are used in parsing command-
 * line arguments
 *****************************************************/
int ParseCmdLine(const int argc, const char* argv[]);
void GetParseError(ostream &os, const char* argv[]);
int set_arg_ext(const char* param, const char* assign, int &count);
int set_arg(const char* param, const char* assign, int &count);

struct CMDLine this_cmd;
/*****************************************************/

void usage(int has_error, char* errmessage) {
	this_cmd.errnum = 0;
	if(has_error) cerr << endl << errmessage << endl ;

	cerr << "Usage: datagen [options]\n"
		 << "  -g, --genome=STRING          Genome .fa file(s)\n"
		 << "  -o, --output=STRING          Output file\n"
		 << "  -t, --output_type=INT        Output type: (default=all)\n"
		 << "                               0-all\n"
		 << "                               1-fasta only\n"
		 << "                               2-fq only\n"
		 << "                               3-fasta and fq only\n"
		 << "  -n, --number=INT             Number of sequences to generate (1000 default)\n"
		 << "  -v, --verbose=INT            Verbose (default=0)\n"
		 << "  -p, --percent=DOUBLE         Probability to use when creating files.\n"
		 << "  -l, --length=INT             Sequence Length\n"
		 << "  -r, --random=NONE            Flag to define a random sequence length\n"
		 << "  -s, --sd=INT                 When -r specified, the standard deviation (default=1).\n"
		 << "  -f, --pos_file=STRING        File containing positions of sequences to use.\n"
		 << "  -E, --equal_space=NONE       Use equal spacing for sequences\n"
		 << endl
		 << "Help options:\n"
		 << "  -?, --help                   Show this help message\n";
	exit(1);
}

inline string get_char_prob(char c, float hit, float mis_hit) {
	ostringstream temp;
	
	if(c == 'n')
		return "0 0 0 0";
	if(c == 'a')
		temp << hit << ' ';
	else
		temp << mis_hit << ' ';
	if(c == 'c')
		temp << hit << ' ';
	else
		temp << mis_hit << ' ';
	if(c == 'g')
		temp << hit << ' ';
	else
		temp << mis_hit << ' ';
	if(c == 't')
		temp << hit << ' ';
	else
		temp << mis_hit << ' ';
		
	return temp.str();
}

inline string get_rand_char_prob(char c, float hit) {
	ostringstream temp;
	double a_rand = rand() % (int)(CHAR_TOTAL - hit);
	double c_rand = rand() % (int)(CHAR_TOTAL - hit);
	double g_rand = rand() % (int)(CHAR_TOTAL - hit);
	double t_rand = rand() % (int)(CHAR_TOTAL - hit);
	if(c == 'n')
		return "0 0 0 0";
	if(c == 'a')
		temp << hit << ' ';
	else
		temp << a_rand << ' ';
	if(c == 'c')
		temp << hit << ' ';
	else
		temp << c_rand << ' ';
	if(c == 'g')
		temp << hit << ' ';
	else
		temp << g_rand << ' ';
	if(c == 't')
		temp << hit << ' ';
	else
		temp << t_rand << ' ';
		
	return temp.str();
}

//get_int will develop the intensity files for the SOL output
inline string get_int(string &str) {
	ostringstream seq_str;
	float hit = perc*CHAR_TOTAL;
	float mis_hit = (CHAR_TOTAL-hit) / 3;
		
	for(unsigned int i=0; i<str.length(); i++) {
		seq_str << get_char_prob(str[i],hit,mis_hit) << "\t";	
	}
	
	return seq_str.str();
}

string get_fq_scores(string &str) {
	ostringstream temp;
	int Q = (int) (perc * 100);
	//printf("Q: %d  CHAR_TOTAL: %d  perc: %f\n",Q,CHAR_TOTAL,perc);	
	char q = (char)((Q<=93 ? Q : 93) + 33);
	
	for(unsigned int i=0; i<str.length(); i++) {
		temp << q;
	}
	
	return temp.str();
}
inline string get_char_qual(char &c, float &hit, float &mis_hit) {
	ostringstream temp;
	int Q_hit = (int)(10 * log( hit / (1-hit) ));
	int Q_mis = (int)(10 * log( mis_hit / (1-mis_hit) ));
	  
	if(c == 'n')
		return "-40 -40 -40 -40";
	if(c == 'a')
		temp << Q_hit << ' ';
	else
		temp << Q_mis << ' ';
	if(c == 'c')
		temp << Q_hit << ' ';
	else
		temp << Q_mis << ' ';
	if(c == 'g')
		temp << Q_hit << ' ';
	else
		temp << Q_mis << ' ';
	if(c == 't')
		temp << Q_hit << ' ';
	else
		temp << Q_mis << ' ';
		
	return temp.str();
}

//Will get the quality scores (_prb.int) file for the SOL data
string get_prb(string &str) {
	ostringstream qual_scores;
	
	float hit = perc*CHAR_TOTAL;
	float hit_prob = hit/CHAR_TOTAL;
	float mis_hit = (CHAR_TOTAL-hit) / 3;
	float mis_hit_prob = mis_hit/CHAR_TOTAL;
	
	for(unsigned int i=0; i<str.length(); i++) {
		qual_scores << get_char_qual(str[i],hit_prob,mis_hit_prob) << "\t";	
	}
	
	return qual_scores.str();
}

int get_norm(int _mean, int _sd) {

	//in order to set the sd for the specific sequence, we use .975 * sd 
	//(for 97.5% of the data to lie within 1sd
	double sd_1 = .975 * _sd;

	// this is to convert it to a decimal between 0 and 1 for use in the ltqnorm function
	double rand_i = (int)(rand() % RAND_HIGH);
	rand_i /= RAND_HIGH;

	double result = ltqnorm((double)rand_i);

	result *= sd_1;
	result += _mean;

	if(result < 1)
		return 1;
	else
		return (int)result;
}

void CreateAll(Genome &gen, valid_seq &vs) {
	char* _int = new char[100+10];
	strcpy(_int,output_file);
	strcat(_int,"_int.txt");
	char* _fa = new char[100+10];
	strcpy(_fa,output_file);
	strcat(_fa,".fasta");
	char* _fq = new char[100+10];
	strcpy(_fq,output_file);
	strcat(_fq,".fq");
	char* _prb = new char[100+10];
	strcpy(_prb,output_file);
	strcat(_prb,"_prb.txt");
	
	ofstream out_fa(_fa);
	ofstream out_prb(_prb);
	ofstream out_fq(_fq);
	ofstream out_int(_int);

	unsigned int to_print = num_gen/50;
	if(VERBOSE)
		cerr << "Creating sequences..." << endl;
		
	unsigned long int gen_size = gen.size();
	unsigned long int gen_space = gen_size / num_gen;

	if(VERBOSE > 1)
		cout << "Size: " << gen_size << endl;
	
	for(unsigned int i=0; i<num_gen; i++) {
		if(VERBOSE && (i % to_print == 0) ) {
			cout << "."; cout.flush();
		}
		
		unsigned long int this_rand;
		unsigned int seq_length = SEQ_LENGTH;
		unsigned int adjust = 0;	//just in case this position won't work, adjust it.
		
		string this_str;
		
		do {
			// If we want to have a random sequence length, we can change the length here
			if(RAND) 
				seq_length = get_norm(SEQ_LENGTH,sd);
		
			//this_rand = (rand() % (gen.size() - seq_length));
			if(EQ_SPACE)	//if we want equal spacing, use this.
				this_rand = (gen_space * i) + adjust++;
			else
				this_rand = (long unsigned int)((gen.size() - seq_length) * (rand() / (RAND_MAX+1.0)));

			if(pos_file != NULL) {
				this_rand = vs.get_next_rand(i);
				//cout << "this_rand: " << this_rand << endl;
				/* string spot = gen.GetPos(this_rand,true);
				while( !vs.is_valid(spot,SEQ_LENGTH) ) {
					this_rand = (rand() % (gen.size() - SEQ_LENGTH));
					spot = gen.GetPos(this_rand,true);
				}
				*/
			}
		
			//Get the string at this position
			this_str = gen.GetString(this_rand,seq_length); 

			if(this_str == "")
				printf("Bad position: %ld\n",this_rand);

		}while(this_str == "");


		if(t_fasta) {
			out_fa << ">" << gen.GetPos(this_rand,true) << " length=" << seq_length << endl;
			out_fa << this_str << endl;
		}

		if(t_int) {
			ostringstream oint;
			oint << "0\t0\t0\t0\t";
			oint << get_int(this_str) << endl;
			out_int << oint.str();
		}

		if(t_fq) {
			ostringstream ofq;
			ofq << "@" << gen.GetPos(this_rand,true) << endl;
			ofq << this_str << endl;
			ofq << "+" << endl;
			ofq << get_fq_scores(this_str) << endl;
			out_fq << ofq.str();
		}

		if(t_prb) {
			ostringstream oprb;
			oprb << get_prb(this_str) << endl;
			out_prb << oprb.str();
		}
	}
		
	if(VERBOSE)
		cerr << "\nOutput written to ";
	if(t_fasta)
		cerr << _fa << " ";
	if(t_fq)
		cerr << _fq << " ";
	if(t_int)
		cerr << _int << " ";
	if(t_prb)
		cerr << _prb << " ";
	cerr << endl;

	out_fa.close();
	out_prb.close();
	out_fq.close();
	out_int.close();

	delete[] _prb;
	delete[] _fa;
	delete[] _fq;
	delete[] _int;

}

void CreateFaFq(Genome &gen, valid_seq &vs) {

	char* _fa = new char[100+10];
	strcpy(_fa,output_file);
	strcat(_fa,".fasta");
	char* _fq = new char[100+10];
	strcpy(_fq,output_file);
	strcat(_fq,".fq");
	
	ofstream out_fa(_fa);
	ofstream out_fq(_fq);

	unsigned int to_print = num_gen/50;
	if(VERBOSE)
		cerr << "Creating sequences..." << endl;
		
	if(VERBOSE > 1)
		cout << "Size: " << gen.size() << endl;

	for(unsigned int i=0; i<num_gen; i++) {
		if(VERBOSE && (i % to_print == 0) ) {
			cout << "."; cout.flush();
		}
		
		unsigned int this_rand;
		unsigned int seq_length = SEQ_LENGTH;
		
		string this_str;

		do {
			// If we want to have a random sequence length, we can change the length here
			if(RAND) 
				seq_length = get_norm(SEQ_LENGTH,sd);
		
			this_rand = (rand() % (gen.size() - seq_length));

			if(pos_file != NULL) {
				this_rand = vs.get_next_rand(i);
				//cout << "this_rand: " << this_rand << endl;
				/* string spot = gen.GetPos(this_rand,true);
				while( !vs.is_valid(spot,SEQ_LENGTH) ) {
					this_rand = (rand() % (gen.size() - SEQ_LENGTH));
					spot = gen.GetPos(this_rand,true);
				}
				*/
			}
		
			//Get the string at this position
			this_str = gen.GetString(this_rand,seq_length);

			if(this_str == "")
				printf("Bad: %s\n",this_str.c_str());

		}while(this_str == "");


		if(t_fasta) {
			out_fa << ">" << gen.GetPos(this_rand,true) << " length=" << seq_length << endl;
			out_fa << this_str << endl;
		}

		if(t_fq) {
			ostringstream ofq;
			ofq << "@" << gen.GetPos(this_rand,true) << endl;
			ofq << this_str << endl;
			ofq << "+" << endl;
			ofq << get_fq_scores(this_str) << endl;
			out_fq << ofq.str();
		}
	}
		
	if(VERBOSE)
		cerr << "\nOutput written to " << _fa << " " << _fq << endl;

	out_fa.close();
	out_fq.close();

	delete[] _fa;
	delete[] _fq;

}

void CreateFa(Genome &gen, valid_seq &vs) {
	char* _fa = new char[100+10];
	strcpy(_fa,output_file);
	strcat(_fa,".fasta");
	
	ofstream out_fa(_fa);

	unsigned int to_print = num_gen/50;
	if(VERBOSE)
		cerr << "Creating sequences..." << endl;
		
	if(VERBOSE > 1)
		cout << "Size: " << gen.size() << endl;

	for(unsigned int i=0; i<num_gen; i++) {
		if(VERBOSE && (i % to_print == 0) ) {
			cout << "."; cout.flush();
		}
		
		unsigned int this_rand;
		unsigned int seq_length = SEQ_LENGTH;
		
		string this_str;

		do {
			// If we want to have a random sequence length, we can change the length here
			if(RAND) 
				seq_length = get_norm(SEQ_LENGTH,sd);
		
			this_rand = (rand() % (gen.size() - seq_length));

			//if there's a position file included, we want to get the information from them.
			if(pos_file != NULL) {

				// vs.get_next_rand() will ensure that each position is equally represented
				// in the random generation.
				this_rand = vs.get_next_rand(i);
				//cout << "this_rand: " << this_rand << endl;

				// This block will take a random sample from the random positions--possibly
				// causing some to be over-expressed
				/* string spot = gen.GetPos(this_rand,true);
				while( !vs.is_valid(spot,SEQ_LENGTH) ) {
					this_rand = (rand() % (gen.size() - SEQ_LENGTH));
					spot = gen.GetPos(this_rand,true);
				}
				*/
			}
		
			//Get the string at this position
			this_str = gen.GetString(this_rand,seq_length);

			if(this_str == "")
				printf("Bad: %s\n",this_str.c_str());

		}while(this_str == "");


		if(t_fasta) {
			out_fa << ">" << gen.GetPos(this_rand,true) << " length=" << seq_length << endl;
			out_fa << this_str << endl;
		}

	}
		
	if(VERBOSE)
		cerr << "\nOutput written to " << _fa << endl;

	out_fa.close();
	delete[] _fa;
}

void CreateFq(Genome &gen, valid_seq &vs) {

	char* _fq = new char[100+10];
	strcpy(_fq,output_file);
	strcat(_fq,".fq");
	
	ofstream out_fq(_fq);

	unsigned int to_print = num_gen/50;
	if(VERBOSE)
		cerr << "Creating sequences..." << endl;
		
	if(VERBOSE > 1)
		cout << "Size: " << gen.size() << endl;

	for(unsigned int i=0; i<num_gen; i++) {
		if(VERBOSE && (i % to_print == 0) ) {
			cout << "."; cout.flush();
		}
		
		unsigned int this_rand;
		unsigned int seq_length = SEQ_LENGTH;
		
		string this_str;

		do {
			// If we want to have a random sequence length, we can change the length here
			if(RAND) 
				seq_length = get_norm(SEQ_LENGTH,sd);
		
			this_rand = (rand() % (gen.size() - seq_length));

			if(pos_file != NULL) {
				this_rand = vs.get_next_rand(i);
				//cout << "this_rand: " << this_rand << endl;
				/* string spot = gen.GetPos(this_rand,true);
				while( !vs.is_valid(spot,SEQ_LENGTH) ) {
					this_rand = (rand() % (gen.size() - SEQ_LENGTH));
					spot = gen.GetPos(this_rand,true);
				}
				*/
			}
		
			//Get the string at this position
			this_str = gen.GetString(this_rand,seq_length);

			if(this_str == "")
				printf("Bad: %s\n",this_str.c_str());

		}while(this_str == "");


		ostringstream ofq;
		ofq << "@" << gen.GetPos(this_rand,true) << endl;
		ofq << this_str << endl;
		ofq << "+" << endl;
		ofq << get_fq_scores(this_str) << endl;
		out_fq << ofq.str();
	}
		
	if(VERBOSE)
		cerr << "\nOutput written to " << _fq << endl;

	out_fq.close();

	delete[] _fq;

}

int main(const int argc, const char* argv[]) {
	
	//Need to set up the ALIGN_SCORES matrix for most of the files...
	for(unsigned int i=0; i<256; i++)
		for(unsigned int j=0; j<4; j++)
			ALIGN_SCORES[i][j] = GAP;
	ALIGN_SCORES[(unsigned int)'a'][0] = MATCH;
	ALIGN_SCORES[(unsigned int)'a'][1] = MISMATCH;
	ALIGN_SCORES[(unsigned int)'a'][2] = MISMATCH;
	ALIGN_SCORES[(unsigned int)'a'][3] = MISMATCH;

	ALIGN_SCORES[(unsigned int)'c'][1] = MATCH;
	ALIGN_SCORES[(unsigned int)'c'][0] = MISMATCH;
	ALIGN_SCORES[(unsigned int)'c'][2] = MISMATCH;
	ALIGN_SCORES[(unsigned int)'c'][3] = MISMATCH;

	ALIGN_SCORES[(unsigned int)'g'][2] = MATCH;
	ALIGN_SCORES[(unsigned int)'g'][0] = MISMATCH;
	ALIGN_SCORES[(unsigned int)'g'][1] = MISMATCH;
	ALIGN_SCORES[(unsigned int)'g'][3] = MISMATCH;

	ALIGN_SCORES[(unsigned int)'t'][3] = MATCH;
	ALIGN_SCORES[(unsigned int)'t'][0] = MISMATCH;
	ALIGN_SCORES[(unsigned int)'t'][1] = MISMATCH;
	ALIGN_SCORES[(unsigned int)'t'][2] = MISMATCH;


	//Print command line args
	for(int i=0; i<argc; i++)
		cout << argv[i] << ' ';
	cout << endl;
	

	int rc = ParseCmdLine(argc,argv);
	if(rc != 0) {
		// an error occurred during option processing
		GetParseError(cerr, argv);
		return -1;
	}

	if(argc < 2) {
		usage(1, "Need at least a genome and a file to map.");
		exit(1);
	}
	if(perc == 0) {
		cerr << "ERROR: Must specify a percentage" << endl;
		cerr << "\tUser specified: " << perc << endl;
		usage(0,NULL);
		exit(1);
	}
	if(SEQ_LENGTH == 0) {
		usage(1,"ERROR: Please use the -l option to specify a sequence length");
		exit(1);
	}
	HASH_SIZE = (unsigned int) pow(4.0,MER_SIZE);

	assert(HASH_SIZE != 0);


	/***********************Check for sd Errors**********************************/
	//Change RAND to true if the sd has been changed (not 1)
	if(sd != 0)
		RAND = 1;
	if(sd < 0) {
		usage(1,"ERROR: Invalid standard deviation value");
		exit(1);
	}
	
	Genome gen;
	gen.use(genome_file);
	gen.store();

	valid_seq vs;
	if(pos_file != NULL) {
		vs.store(pos_file);
	}
	
	/********************ERROR CHECKING FOR OUTPT_TYPE**************************/
	switch(output_type) {
		case 1:
			t_fasta=true;
			t_fq=false;
			t_int=false;
			t_prb=false;
			CreateFa(gen,vs);
			break;
		case 2:
			t_fasta=false;
			t_fq=true;
			t_int=false;
			t_prb=false;
			CreateFq(gen,vs);
			break;
		case 3:
			t_fasta=true;
			t_fq=true;
			t_int=false;
			t_prb=false;
			CreateFaFq(gen,vs);
			break;
		case 0:	// this is the default - do nothing because they're already set to true
		default:
			CreateAll(gen,vs);
			break;
	}
	/****************************************************************************/
	
	return 0;
}

int ParseCmdLine(const int argc, const char* argv[]) {
	
	for(int i=1; i<argc; i++) {
		if(*argv[i] == '-') { //check for normal
			int set_ret;

			if(*((argv[i])+1) == '-')  { //check for extended
				//printf("Found extended\n");
				set_ret = set_arg_ext(argv[i],argv[i+1],i);
			}
			else {
				set_ret = set_arg(argv[i],argv[i+1],i);
				i++;	//increment past the next expression
			}
			
			if(set_ret != 0) {
				this_cmd.errnum = set_ret;
				this_cmd.errpos = i;
				return -1;
			}

		}
		else {
			this_cmd.errnum = INVALID_ARG;
			this_cmd.errpos = i;
			return INVALID_ARG;
		}
	}

	return 0;

}

void GetParseError(ostream &os, const char* argv[]) {
	if(this_cmd.errnum == PARSE_ERROR) {
		os << "Irregular Parameter in: " << argv[this_cmd.errpos] << endl;
	}
	else if(this_cmd.errnum == NO_MATCHING_ARG) {
		os << "No matching arg in: " << argv[this_cmd.errpos] << endl;
	}
	else if(this_cmd.errnum == INVALID_ARG) {
		os << "There were unused arguments.  Did you forget to specify multiple files in \"quotations\" ("
		   << argv[this_cmd.errpos] << ")" << endl;
	}
	else {
		os << "Unknown error " << this_cmd.errnum << " in: " << argv[this_cmd.errpos] << endl;
	}

}


int set_arg(const char* param, const char* assign, int &count) {
	switch(*(param+1)) {	//switch on the first character
		case 'g':
			genome_file = assign;
			break;
		case 'o':
			output_file = assign;
			break;
		case 't':
			if(sscanf(assign,"%d",&output_type) < 1)
				return PARSE_ERROR;
			break;
		case 'n':
			if(sscanf(assign,"%d",&num_gen) < 1)
				return PARSE_ERROR;
			break;
		case 'v':
			if(sscanf(assign,"%d",&VERBOSE) < 1)
				return PARSE_ERROR;
			break;
		case 'p': 
			if(sscanf(assign,"%lf",&perc) < 1)
				return PARSE_ERROR;
			break;
		case 'l':
			if(sscanf(assign,"%d",&SEQ_LENGTH) < 1)
				return PARSE_ERROR;
			break;
		case 'r':
			RAND = true;
			count--;
			break;
		case 's':
			if(sscanf(assign,"%d",&sd) < 1)
				return PARSE_ERROR;
			break;
		case 'f':
			pos_file = assign;
			break;
		case 'E':
			EQ_SPACE = 1;
			count--;
			break;
		default:
			return NO_MATCHING_ARG;
	}

	return 0;

}


int set_arg_ext(const char* param, const char* assign, int &count) {
	char which=' ';
	int adjust=0;

	if(strncmp(param+2,"genome",6) == 0) {	//check to see if it's the genome
		which = 'g';
		adjust = 8;	//the size of "genome" plus 2
	}
	else if(strncmp(param+2,"output=",7) == 0) {
		//need to differentiate between output and output_type.  that's why it's 7
		which = 'o';
		adjust = 8;
	}
	else if(strncmp(param+2,"output_type",11) == 0) {
		which = 't';
		adjust = 13;
	}
	else if(strncmp(param+2,"number",6) == 0) {
		which = 'n';
		adjust = 8;
	}
	else if(strncmp(param+2,"verbose",7) == 0) {
		which = 'v';
		adjust = 9;
	}
	else if(strncmp(param+2,"percent",7) == 0) {
		which = 'p';
		adjust = 9;
	}
	else if(strncmp(param+2,"length",6) == 0) {
		which = 'l';
		adjust = 8;
	}
	else if(strncmp(param+2,"random",6) == 0) {
		which = 'r';
		adjust = 8;
	}
	else if(strncmp(param+2,"sd",2) == 0) {
		which = 's';
		adjust = 4;
	}

	else if(strncmp(param+2,"pos_file",9) == 0) {
		which = 'f';
		adjust = 11;
	}
	else if(strcmp(param+2,"help") == 0) {
		usage(0,""); //usage will exit immediately
	}
	else if(strcmp(param+2,"equal_space") == 0) {
		which = 'E';
		adjust = 13;	//do I need this?
	}
	else 
		return NO_MATCHING_ARG;

	param += ++adjust;

	switch(which) {	//switch on the first character
		case 'g':
			genome_file = param;
			break;
		case 'o':
			output_file = param;
			break;
		case 't':
			if(sscanf(param,"%d",&output_type) < 1)
				return PARSE_ERROR;
			break;
		case 'n':
			if(sscanf(param,"%d",&num_gen) < 1)
				return PARSE_ERROR;
			break;
		case 'v':
			if(sscanf(param,"%d",&VERBOSE) < 1)
				return PARSE_ERROR;
			break;
		case 'p': 
			if(sscanf(param,"%lf",&perc) < 1)
				return PARSE_ERROR;
			break;
		case 'l':
			if(sscanf(param,"%d",&SEQ_LENGTH) < 1)
				return PARSE_ERROR;
			break;
		case 'r':
			RAND = true;
			break;
		case 's':
			if(sscanf(param,"%d",&sd) < 1)
				return PARSE_ERROR;
			break;
		case 'f':
			pos_file = param;
			break;
		case 'E':
			EQ_SPACE = true;
			break;
		default:
			return NO_MATCHING_ARG;
	}

	return 0;

}
